@@ -2,7 +2,11 @@ package Reddit::Client;
 
 # Remember to include new version and date in the docs, or get rid of that 
 # section altogether
-our $VERSION = '1.312'; # 4/2 1.312
+our $VERSION = '1.32'; # 5/29 1.32 unban
+# 5/3 .315 submit_comment now returns fullname not id
+# 4/25 .314 4/8 1.313
+#  .314 added locked key to Comment, was this a recent Reddit change?
+# 1.313 changed the behavior or print_request_errors
 # 1.312 requests that fail with print_request_errors as true now die instead of
 # 		croak, which lets you capture the error message
 # 1.31 added a bunch of flags to Link, Thing, Comment, SubReddit
@@ -130,6 +134,7 @@ use constant API_MORECHILDREN        => 40;
 use constant API_BY_ID               => 41;
 use constant API_FLAIR               => 42;
 use constant API_DELETEFLAIR         => 43;
+use constant API_UNBAN               => 44;
 
 #===============================================================================
 # Parameters
@@ -187,6 +192,7 @@ $API[API_MORECHILDREN  ] = ['GET',  '/api/morechildren'       ];
 $API[API_BY_ID         ] = ['GET',  '/by_id'                  ];
 $API[API_FLAIR         ] = ['POST', '/r/%s/api/flair'         ];
 $API[API_DELETEFLAIR   ] = ['POST', '/r/%s/api/deleteflair'   ];
+$API[API_UNBAN         ] = ['POST', '/r/%s/api/unfriend'      ];
 
 #===============================================================================
 # Class methods
@@ -852,6 +858,25 @@ sub ban {
 	);
 	return $result;
 }
+
+sub unban {
+	my ($self, %param) = @_;
+	my $sub	= $param{sub} || $param{subreddit} || die "subreddit is required\n";
+	
+	my $data = {}; 
+	$data->{name}	= $param{username} || die "username is required\n";
+	# ban_context = fullname, but of what - not required
+
+	$data->{type} = 'banned';
+
+	my $result = $self->api_json_request(
+		api  => API_UNBAN, 
+		args => [$sub],
+		data => $data,
+	);
+	return $result;
+}
+
 sub get_modlinks {
     my ($self, %param) = @_;
 
@@ -1381,7 +1406,7 @@ sub submit_comment {
 		#sendreplies=>$replies,
     });
 
-    return $result->{data}{things}[0]{data}{id};
+    return $result->{data}{things}[0]{data}{name};
 }
 
 sub comment {
@@ -1519,6 +1544,7 @@ sub create_multi {
         	if (ref $param{subreddits} ne 'ARRAY') { croak "'subreddits' must be an array reference."; }
 
 		$model->{subreddits} = [ map { { name=> $_ } } @{$param{subreddits}} ];
+		#print Dumper($model->{subreddits});
 	}
 
 	# Put a ribbon on it
@@ -1639,10 +1665,6 @@ __END__
 
 Reddit::Client - A Perl wrapper for the Reddit API.
 
-=head1 VERSION
-
-1.311 3/30/19
-
 =head1 DESCRIPTION
 
 Reddit::Client handles Oauth session management and HTTP communication with Reddit's external API. For more information about the Reddit API, see L<https://github.com/reddit/reddit/wiki/API>. 
@@ -1804,10 +1826,13 @@ This function is an alias for C<submit_comment>, and is equivalent to
 
     create_multi ( name => $multi_name, 
                  [ description => $description, ] [ visibility => $visibility, ] [ subreddits => [ subreddits ], ]
-                 [ icon_name => $icon_name, ] [ key_color => $hex_code, ] [ weighting_scheme => $weighting_scheme, ] )
+                 [ icon_name => $icon_name, ] [ key_color => $hex_code, ] [ weighting_scheme => $weighting_scheme, ] 
+                 [ username => $username, ] )
 
 Create a multireddit. The only required argument is the name. A multi can also be created with C<edit_multi>, the only difference being that C<create_multi> will fail with a HTTP 409 error if a multi with that name already exists. As of March 2019, trying to add a banned sub to a multi will fail with a 403 Unauthorized.
 
+Requires a username, which script apps have by default, but if you're using a web app, you'll need to either pass it in explicitly, or set the username property on your Reddit::Client object. 
+
 Returns a hash of information about the newly created multireddit.
 
 =over
@@ -1948,7 +1973,7 @@ or
 
     get_comments ( url => $url )
 
-Get the comment tree for the selected subreddit/link\_id, subreddit/link\_id/comment_id, permalink, or URL. This will be a mix of Comment and MoreComments objects, which are placeholders for collapsed comments. This is analogous to the "show more comments" links on the website.
+Get the comment tree for the selected subreddit/link_id, subreddit/link_id/comment_id, permalink, or URL. This will be a mix of Comment and MoreComments objects, which are placeholders for collapsed comments. This is analogous to the "show more comments" links on the website.
 
 If you already have a Link or Comment object, it's best to call its own C<get_comments> method, which takes no arguments and supplies all of the necessary information for you. If you do decide to use this version:
 
@@ -2098,7 +2123,7 @@ or
 
     get_token
 
-Get an authentication token from Reddit. Normally a user has no reason to call this function themselves. If you pass in your authentication info when creating a new Reddit::Client onject, C<get_token> will be called automatically using the information provided. Similarly, if your script runs continuously for more than an hour, a new token will be obtained automatically. C<get_token> is exposed in case you need to refresh your authorization token manually for some reason, for example if you want to switch to a different user within the same Reddit::Client instance.
+Get an authentication token from Reddit. Normally a user has no reason to call this function themselves. If you pass in your authentication info when creating a new Reddit::Client onject, C<get_token> will be called automatically using the information provided. If your script runs continuously for more than an hour, a new token will be obtained automatically. C<get_token> is exposed in case you need to refresh your authorization token manually for some reason, for example if you want to switch to a different user within the same Reddit::Client instance.
 
 If any arguments are provided, all of the appropriate arguments are required. If none are provided, it will use the information from the previous call.
 
@@ -2108,7 +2133,7 @@ If any arguments are provided, all of the appropriate arguments are required. If
 			   
 Get information about a user, where C<view> is one of the user L<constants|https://metacpan.org/pod/Reddit::Client#CONSTANTS>: overview, comments, submitted, gilded, upvoted, downvoted, hidden, saved, or about. Defaults to 'overview', which shows the user's most recent comments and posts.
 
-The result will be a listing of Links or Comments or a mix of both, except in the case of the 'about' view, in which case it will be a single Account object.
+The result will be a listing of Links and/or Comments, except in the 'about' view, in which case it will be a single Account object.
 
 =item get_wiki
 
@@ -2140,7 +2165,7 @@ Hide a post.
 
     ignore_reports ( $fullname )
 
-	Ignore reports for a comment or post (moderator action).
+Ignore reports for a comment or post (moderator action).
 
 =item info 
 
@@ -2178,21 +2203,29 @@ or
     new ( user_agent => $user_agent, 
         [ client_id => $client_id, secret => $secret, refresh_token => $refresh_token ]
         [ print_request_errors => 0, ]  [ print_response => 0, ] [ print_request => 0, ] [ print_request_on_error => 0 ]
-        [ subdomain => 'www', ] )
+        [ subdomain => 'www', ] [ username => $username ] )
 
 Instantiate a new Reddit::Client object. Optionally authenticate at the same time. (Unless you have some reason not to, this is the recommended way to do it.) For "script"-type apps, this is done by passing in a username, password, client_id and secret. For "web"-type apps, this is done by passing in a refresh_token, client_id and secret.
 
-C<user_agent> is a string that uniquely identifies your app. The API Rules (L<https://github.com/reddit/reddit/wiki/API#rules>) say it should be "something unique and descriptive, including the target platform, a unique application identifier, a version string, and your username as contact information". It also includes this warning: "NEVER lie about your user-agent. This includes spoofing popular browsers and spoofing other bots. We will ban liars with extreme prejudice." C<user_agent> is required as of version 1.2 (before, Reddit::Client would provide one if you didn't).
+C<user_agent> is a string that uniquely identifies your app. The L<API rules|https://github.com/reddit/reddit/wiki/API#rules>) say it should be "something unique and descriptive, including the target platform, a unique application identifier, a version string, and your username as contact information". It also includes this warning: "NEVER lie about your user-agent. This includes spoofing popular browsers and spoofing other bots. We will ban liars with extreme prejudice." C<user_agent> is required as of version 1.2 (before, Reddit::Client would provide one if you didn't).
 
-Optional arguments:
+C<subdomain> is the subdomain in links generated by Reddit::Client (for example with C<get_web_url>). You can use this to generate links to old.reddit.com to force the old version of Reddit, for example, or new.reddit.com for the new. Default www.
+
+C<username> is optional for web apps. Unlike a script app, at no point does a web app know your username unless you explicitly provide it. This means that if you're using a function that requires a username (L<create_multi|https://metacpan.org/pod/Reddit::Client#create_multi> and L<edit_multi|https://metacpan.org/pod/Reddit::Client#edit_multi> are two), and you haven't either passed it into the function directly or set the property in your Reddit::Client object, it will fail. 
+
+B<Error handling>
+
+By default, if there is an error, Reddit::Client will print the HTTP status line and then die. You can change this behavior with the following variables:
 
 =over
 
-C<subdomain>: The subdomain in links generated by Reddit::Client (for example with C<get_web_url>). You can use this to generate links to old.reddit.com to force the old version of Reddit, for example, or new.reddit.com for the new. Default www.
+C<print_request_errors>: If there was an error, print some information about it before dying.
 
-C<print_response_content>: Print the content portion of Reddit's HTTP response. Default is print nothing on success and an error code on failure. The content will usually be a blob of JSON, but for certain 500 errors, it may be an entre web page.
+Reddit will usually return some JSON in the case of an error. If it has, Reddit::Client will add some of its own information to it, encode it all to a JSON string, print it, and die. It will contain the keys C<code>, C<status_line>, C<error> (which will always be 1), and C<data>, which will contain Reddit's JSON data. The fields in Reddit's return JSON are unpredictable and vary from endpoint to endpoint.
 
-C<print_request_errors>: If there is an error, print the content portion of Reddit's response. Not very useful as Reddit's response is usually just a text string repeating the error code.
+Sometimes Reddit will not return valid JSON; for example, if the request fails because Reddit's CDN was unable to reach their servers, you'll get a complete webpage (which is actually the same page you would see if you'd attempted the request with a browser). If Reddit did not return valid JSON for this or some other reason, Reddit::Client will print the HTTP status line and the content portion of the response.
+
+C<print_response_content>: Print the content portion of Reddit's HTTP response for every request, whether it succeeded or not. 
 
 C<print_request>: Print the I<entire> HTTP request and response for every request.
 
@@ -2274,6 +2307,12 @@ If C<inbox_replies> is defined and is false, disable inbox replies for that post
 
 Submit a text post. Returns the fullname of the new post. If C<inbox_replies> is defined and is false, disable inbox replies for that post.
 
+=item unban
+
+    unban ( username => $username, subreddit => $subreddit )
+
+Un-ban a user (moderator action).
+
 =item unhide 
 
     unhide ( $fullname )
